@@ -25,23 +25,32 @@
 
 #pragma once
 
+// FIXME: Should rename this file AXCoreObject.h.
+
 #include "FrameLoaderClient.h"
 #include "HTMLTextFormControlElement.h"
 #include "LayoutRect.h"
-#include "Range.h"
 #include "TextIterator.h"
-#include "VisiblePosition.h"
 #include "VisibleSelection.h"
+#include "Widget.h"
 #include <wtf/RefCounted.h>
+#include <wtf/Variant.h>
 
 #if PLATFORM(WIN)
 #include "AccessibilityObjectWrapperWin.h"
 #include "COMPtr.h"
 #endif
 
+#if USE(ATK)
+#include <wtf/glib/GRefPtr.h>
+#endif
+
 #if PLATFORM(COCOA)
 OBJC_CLASS WebAccessibilityObjectWrapper;
 typedef WebAccessibilityObjectWrapper AccessibilityObjectWrapper;
+typedef struct _NSRange NSRange;
+typedef const struct __AXTextMarker* AXTextMarkerRef;
+typedef const struct __AXTextMarkerRange* AXTextMarkerRangeRef;
 #elif USE(ATK)
 typedef struct _WebKitAccessible WebKitAccessible;
 typedef struct _WebKitAccessible AccessibilityObjectWrapper;
@@ -55,24 +64,25 @@ class TextStream;
 
 namespace WebCore {
 
-class Node;
-class Element;
-class RenderObject;
-class QualifiedName;
-class Path;
-class Widget;
-class Page;
+class AXCoreObject;
+class AXObjectCache;
+class AccessibilityScrollView;
 class Document;
+class Element;
 class Frame;
 class FrameView;
+class Node;
+class Page;
+class Path;
+class QualifiedName;
+class RenderObject;
 class ScrollView;
-class AXObjectCache;
-class AXCoreObject;
-class AccessibilityScrollView;
+class Widget;
+
 struct AccessibilityText;
 struct ScrollRectToVisibleOptions;
 
-typedef size_t AXID;
+using AXID = size_t;
 extern const AXID InvalidAXID;
 
 enum class AccessibilityRole {
@@ -229,7 +239,317 @@ enum class AccessibilityRole {
     Window,
 };
 
-enum class AccessibilityDetachmentType { CacheDestroyed, ElementDestroyed, ElementChange };
+ALWAYS_INLINE String accessibilityRoleToString(AccessibilityRole role)
+{
+    switch (role) {
+    case AccessibilityRole::Annotation:
+        return "Annotation";
+    case AccessibilityRole::Application:
+        return "Application";
+    case AccessibilityRole::ApplicationAlert:
+        return "ApplicationAlert";
+    case AccessibilityRole::ApplicationAlertDialog:
+        return "ApplicationAlertDialog";
+    case AccessibilityRole::ApplicationDialog:
+        return "ApplicationDialog";
+    case AccessibilityRole::ApplicationGroup:
+        return "ApplicationGroup";
+    case AccessibilityRole::ApplicationLog:
+        return "ApplicationLog";
+    case AccessibilityRole::ApplicationMarquee:
+        return "ApplicationMarquee";
+    case AccessibilityRole::ApplicationStatus:
+        return "ApplicationStatus";
+    case AccessibilityRole::ApplicationTextGroup:
+        return "ApplicationTextGroup";
+    case AccessibilityRole::ApplicationTimer:
+        return "ApplicationTimer";
+    case AccessibilityRole::Audio:
+        return "Audio";
+    case AccessibilityRole::Blockquote:
+        return "Blockquote";
+    case AccessibilityRole::Browser:
+        return "Browser";
+    case AccessibilityRole::BusyIndicator:
+        return "BusyIndicator";
+    case AccessibilityRole::Button:
+        return "Button";
+    case AccessibilityRole::Canvas:
+        return "Canvas";
+    case AccessibilityRole::Caption:
+        return "Caption";
+    case AccessibilityRole::Cell:
+        return "Cell";
+    case AccessibilityRole::CheckBox:
+        return "CheckBox";
+    case AccessibilityRole::ColorWell:
+        return "ColorWell";
+    case AccessibilityRole::Column:
+        return "Column";
+    case AccessibilityRole::ColumnHeader:
+        return "ColumnHeader";
+    case AccessibilityRole::ComboBox:
+        return "ComboBox";
+    case AccessibilityRole::Definition:
+        return "Definition";
+    case AccessibilityRole::Deletion:
+        return "Deletion";
+    case AccessibilityRole::DescriptionList:
+        return "DescriptionList";
+    case AccessibilityRole::DescriptionListTerm:
+        return "DescriptionListTerm";
+    case AccessibilityRole::DescriptionListDetail:
+        return "DescriptionListDetail";
+    case AccessibilityRole::Details:
+        return "Details";
+    case AccessibilityRole::Directory:
+        return "Directory";
+    case AccessibilityRole::DisclosureTriangle:
+        return "DisclosureTriangle";
+    case AccessibilityRole::Div:
+        return "Div";
+    case AccessibilityRole::Document:
+        return "Document";
+    case AccessibilityRole::DocumentArticle:
+        return "DocumentArticle";
+    case AccessibilityRole::DocumentMath:
+        return "DocumentMath";
+    case AccessibilityRole::DocumentNote:
+        return "DocumentNote";
+    case AccessibilityRole::Drawer:
+        return "Drawer";
+    case AccessibilityRole::EditableText:
+        return "EditableText";
+    case AccessibilityRole::Feed:
+        return "Feed";
+    case AccessibilityRole::Figure:
+        return "Figure";
+    case AccessibilityRole::Footer:
+        return "Footer";
+    case AccessibilityRole::Footnote:
+        return "Footnote";
+    case AccessibilityRole::Form:
+        return "Form";
+    case AccessibilityRole::GraphicsDocument:
+        return "GraphicsDocument";
+    case AccessibilityRole::GraphicsObject:
+        return "GraphicsObject";
+    case AccessibilityRole::GraphicsSymbol:
+        return "GraphicsSymbol";
+    case AccessibilityRole::Grid:
+        return "Grid";
+    case AccessibilityRole::GridCell:
+        return "GridCell";
+    case AccessibilityRole::Group:
+        return "Group";
+    case AccessibilityRole::GrowArea:
+        return "GrowArea";
+    case AccessibilityRole::Heading:
+        return "Heading";
+    case AccessibilityRole::HelpTag:
+        return "HelpTag";
+    case AccessibilityRole::HorizontalRule:
+        return "HorizontalRule";
+    case AccessibilityRole::Ignored:
+        return "Ignored";
+    case AccessibilityRole::Inline:
+        return "Inline";
+    case AccessibilityRole::Image:
+        return "Image";
+    case AccessibilityRole::ImageMap:
+        return "ImageMap";
+    case AccessibilityRole::ImageMapLink:
+        return "ImageMapLink";
+    case AccessibilityRole::Incrementor:
+        return "Incrementor";
+    case AccessibilityRole::Insertion:
+        return "Insertion";
+    case AccessibilityRole::Label:
+        return "Label";
+    case AccessibilityRole::LandmarkBanner:
+        return "LandmarkBanner";
+    case AccessibilityRole::LandmarkComplementary:
+        return "LandmarkComplementary";
+    case AccessibilityRole::LandmarkContentInfo:
+        return "LandmarkContentInfo";
+    case AccessibilityRole::LandmarkDocRegion:
+        return "LandmarkDocRegion";
+    case AccessibilityRole::LandmarkMain:
+        return "LandmarkMain";
+    case AccessibilityRole::LandmarkNavigation:
+        return "LandmarkNavigation";
+    case AccessibilityRole::LandmarkRegion:
+        return "LandmarkRegion";
+    case AccessibilityRole::LandmarkSearch:
+        return "LandmarkSearch";
+    case AccessibilityRole::Legend:
+        return "Legend";
+    case AccessibilityRole::Link:
+        return "Link";
+    case AccessibilityRole::List:
+        return "List";
+    case AccessibilityRole::ListBox:
+        return "ListBox";
+    case AccessibilityRole::ListBoxOption:
+        return "ListBoxOption";
+    case AccessibilityRole::ListItem:
+        return "ListItem";
+    case AccessibilityRole::ListMarker:
+        return "ListMarker";
+    case AccessibilityRole::Mark:
+        return "Mark";
+    case AccessibilityRole::MathElement:
+        return "MathElement";
+    case AccessibilityRole::Matte:
+        return "Matte";
+    case AccessibilityRole::Menu:
+        return "Menu";
+    case AccessibilityRole::MenuBar:
+        return "MenuBar";
+    case AccessibilityRole::MenuButton:
+        return "MenuButton";
+    case AccessibilityRole::MenuItem:
+        return "MenuItem";
+    case AccessibilityRole::MenuItemCheckbox:
+        return "MenuItemCheckbox";
+    case AccessibilityRole::MenuItemRadio:
+        return "MenuItemRadio";
+    case AccessibilityRole::MenuListPopup:
+        return "MenuListPopup";
+    case AccessibilityRole::MenuListOption:
+        return "MenuListOption";
+    case AccessibilityRole::Meter:
+        return "Meter";
+    case AccessibilityRole::Outline:
+        return "Outline";
+    case AccessibilityRole::Paragraph:
+        return "Paragraph";
+    case AccessibilityRole::PopUpButton:
+        return "PopUpButton";
+    case AccessibilityRole::Pre:
+        return "Pre";
+    case AccessibilityRole::Presentational:
+        return "Presentational";
+    case AccessibilityRole::ProgressIndicator:
+        return "ProgressIndicator";
+    case AccessibilityRole::RadioButton:
+        return "RadioButton";
+    case AccessibilityRole::RadioGroup:
+        return "RadioGroup";
+    case AccessibilityRole::RowHeader:
+        return "RowHeader";
+    case AccessibilityRole::Row:
+        return "Row";
+    case AccessibilityRole::RowGroup:
+        return "RowGroup";
+    case AccessibilityRole::RubyBase:
+        return "RubyBase";
+    case AccessibilityRole::RubyBlock:
+        return "RubyBlock";
+    case AccessibilityRole::RubyInline:
+        return "RubyInline";
+    case AccessibilityRole::RubyRun:
+        return "RubyRun";
+    case AccessibilityRole::RubyText:
+        return "RubyText";
+    case AccessibilityRole::Ruler:
+        return "Ruler";
+    case AccessibilityRole::RulerMarker:
+        return "RulerMarker";
+    case AccessibilityRole::ScrollArea:
+        return "ScrollArea";
+    case AccessibilityRole::ScrollBar:
+        return "ScrollBar";
+    case AccessibilityRole::SearchField:
+        return "SearchField";
+    case AccessibilityRole::Sheet:
+        return "Sheet";
+    case AccessibilityRole::Slider:
+        return "Slider";
+    case AccessibilityRole::SliderThumb:
+        return "SliderThumb";
+    case AccessibilityRole::SpinButton:
+        return "SpinButton";
+    case AccessibilityRole::SpinButtonPart:
+        return "SpinButtonPart";
+    case AccessibilityRole::SplitGroup:
+        return "SplitGroup";
+    case AccessibilityRole::Splitter:
+        return "Splitter";
+    case AccessibilityRole::StaticText:
+        return "StaticText";
+    case AccessibilityRole::Subscript:
+        return "Subscript";
+    case AccessibilityRole::Summary:
+        return "Summary";
+    case AccessibilityRole::Superscript:
+        return "Superscript";
+    case AccessibilityRole::Switch:
+        return "Switch";
+    case AccessibilityRole::SystemWide:
+        return "SystemWide";
+    case AccessibilityRole::SVGRoot:
+        return "SVGRoot";
+    case AccessibilityRole::SVGText:
+        return "SVGText";
+    case AccessibilityRole::SVGTSpan:
+        return "SVGTSpan";
+    case AccessibilityRole::SVGTextPath:
+        return "SVGTextPath";
+    case AccessibilityRole::TabGroup:
+        return "TabGroup";
+    case AccessibilityRole::TabList:
+        return "TabList";
+    case AccessibilityRole::TabPanel:
+        return "TabPanel";
+    case AccessibilityRole::Tab:
+        return "Tab";
+    case AccessibilityRole::Table:
+        return "Table";
+    case AccessibilityRole::TableHeaderContainer:
+        return "TableHeaderContainer";
+    case AccessibilityRole::Term:
+        return "Term";
+    case AccessibilityRole::TextArea:
+        return "TextArea";
+    case AccessibilityRole::TextField:
+        return "TextField";
+    case AccessibilityRole::TextGroup:
+        return "TextGroup";
+    case AccessibilityRole::Time:
+        return "Time";
+    case AccessibilityRole::Tree:
+        return "Tree";
+    case AccessibilityRole::TreeGrid:
+        return "TreeGrid";
+    case AccessibilityRole::TreeItem:
+        return "TreeItem";
+    case AccessibilityRole::ToggleButton:
+        return "ToggleButton";
+    case AccessibilityRole::Toolbar:
+        return "Toolbar";
+    case AccessibilityRole::Unknown:
+        return "Unknown";
+    case AccessibilityRole::UserInterfaceTooltip:
+        return "UserInterfaceTooltip";
+    case AccessibilityRole::ValueIndicator:
+        return "ValueIndicator";
+    case AccessibilityRole::Video:
+        return "Video";
+    case AccessibilityRole::WebApplication:
+        return "WebApplication";
+    case AccessibilityRole::WebArea:
+        return "WebArea";
+    case AccessibilityRole::WebCoreLink:
+        return "WebCoreLink";
+    case AccessibilityRole::Window:
+        return "Window";
+    }
+    UNREACHABLE();
+    return "";
+}
+
+enum class AccessibilityDetachmentType { CacheDestroyed, ElementDestroyed, ElementChanged };
 
 enum class AccessibilityConversionSpace { Screen, Page };
 
@@ -296,6 +616,8 @@ enum class AccessibilitySearchKey {
     VisitedLink,
 };
 
+using AXEditingStyleValueVariant = Variant<String, bool, int>;
+
 struct AccessibilitySearchCriteria {
     AXCoreObject* anchorObject { nullptr };
     AXCoreObject* startObject;
@@ -371,7 +693,7 @@ enum class AccessibilityTextOperationType {
 };
 
 struct AccessibilityTextOperation {
-    Vector<RefPtr<Range>> textRanges; // text on which perform the operation.
+    Vector<SimpleRange> textRanges; // text on which perform the operation.
     AccessibilityTextOperationType type;
     String replacementText; // For type = replace.
 
@@ -404,10 +726,7 @@ struct AccessibilityTextUnderElementMode {
     { }
 };
 
-#if PLATFORM(COCOA)
-typedef struct _NSRange NSRange;
-#endif
-
+// FIXME: Merge this with CharacterRange (by deleting this and using CharacterRange instead).
 struct PlainTextRange {
     unsigned start { 0 };
     unsigned length { 0 };
@@ -431,24 +750,6 @@ enum class AccessibilityVisiblePositionForBounds {
     Last,
 };
 
-struct VisiblePositionRange {
-    VisiblePosition start;
-    VisiblePosition end;
-
-    VisiblePositionRange() = default;
-    VisiblePositionRange(const VisiblePosition& s, const VisiblePosition& e)
-        : start(s)
-        , end(e)
-    { }
-
-    VisiblePositionRange(const VisibleSelection& selection)
-        : start(selection.start())
-        , end(selection.end())
-    { }
-
-    bool isNull() const { return start.isNull() || end.isNull(); }
-};
-
 enum class AccessibilityMathScriptObjectType { Subscript, Superscript };
 enum class AccessibilityMathMultiscriptObjectType { PreSubscript, PreSuperscript, PostSubscript, PostSuperscript };
 
@@ -495,6 +796,7 @@ public:
     virtual bool isAccessibilityTableInstance() const = 0;
     virtual bool isAccessibilityTableColumnInstance() const = 0;
     virtual bool isAccessibilityProgressIndicatorInstance() const = 0;
+    virtual bool isAccessibilityListBoxInstance() const = 0;
     virtual bool isAXIsolatedObjectInstance() const = 0;
 
     virtual bool isAttachmentElement() const = 0;
@@ -513,7 +815,6 @@ public:
     bool isCheckbox() const { return roleValue() == AccessibilityRole::CheckBox; }
     bool isRadioButton() const { return roleValue() == AccessibilityRole::RadioButton; }
     bool isListBox() const { return roleValue() == AccessibilityRole::ListBox; }
-    virtual bool isNativeListBox() const { return false; };
     virtual bool isListBoxOption() const = 0;
     virtual bool isAttachment() const = 0;
     virtual bool isMediaTimeline() const = 0;
@@ -613,6 +914,8 @@ public:
     bool isTreeItem() const { return roleValue() == AccessibilityRole::TreeItem; }
     bool isScrollbar() const { return roleValue() == AccessibilityRole::ScrollBar; }
     virtual bool isButton() const = 0;
+    
+    virtual HashMap<String, AXEditingStyleValueVariant> resolvedEditingStyles() const = 0;
 
     bool isListItem() const { return roleValue() == AccessibilityRole::ListItem; }
     bool isCheckboxOrRadio() const { return isCheckbox() || isRadioButton(); }
@@ -656,14 +959,15 @@ public:
     virtual void setIsExpanded(bool) = 0;
     virtual FloatRect relativeFrame() const = 0;
     virtual FloatRect convertFrameToSpace(const FloatRect&, AccessibilityConversionSpace) const = 0;
-
+    virtual bool supportsCheckedState() const = 0;
+    
     // In a multi-select list, many items can be selected but only one is active at a time.
     virtual bool isSelectedOptionActive() const = 0;
 
     virtual bool hasBoldFont() const = 0;
     virtual bool hasItalicFont() const = 0;
     virtual bool hasMisspelling() const = 0;
-    virtual RefPtr<Range> getMisspellingRange(RefPtr<Range> const& start, AccessibilitySearchDirection) const = 0;
+    virtual Optional<SimpleRange> misspellingRange(const SimpleRange& start, AccessibilitySearchDirection) const = 0;
     virtual bool hasPlainText() const = 0;
     virtual bool hasSameFont(const AXCoreObject&) const = 0;
     virtual bool hasSameFontColor(const AXCoreObject&) const = 0;
@@ -680,7 +984,7 @@ public:
     virtual bool canSetValueAttribute() const = 0;
     virtual bool canSetNumericValue() const = 0;
     virtual bool canSetSelectedAttribute() const = 0;
-    virtual bool canSetSelectedChildrenAttribute() const = 0;
+    virtual bool canSetSelectedChildren() const = 0;
     virtual bool canSetExpandedAttribute() const = 0;
 
     virtual Element* element() const = 0;
@@ -707,6 +1011,9 @@ public:
     virtual AXCoreObject* selectedListItem() = 0;
     virtual int layoutCount() const = 0;
     virtual double estimatedLoadingProgress() const = 0;
+    virtual String brailleLabel() const = 0;
+    virtual String brailleRoleDescription() const = 0;
+    virtual String embeddedImageDescription() const = 0;
 
     virtual bool supportsARIAOwns() const = 0;
     virtual bool isActiveDescendantOfFocusedContainer() const = 0;
@@ -736,6 +1043,7 @@ public:
     virtual bool supportsPressed() const = 0;
     virtual bool supportsExpanded() const = 0;
     virtual bool supportsChecked() const = 0;
+    virtual bool supportsRowCountChange() const = 0;
     virtual AccessibilitySortDirection sortDirection() const = 0;
     virtual bool canvasHasFallbackContent() const = 0;
     virtual bool supportsRangeValue() const = 0;
@@ -785,7 +1093,7 @@ public:
     virtual bool isDescendantOfRole(AccessibilityRole) const = 0;
 
     // Text selection
-    virtual Vector<RefPtr<Range>> findTextRanges(AccessibilitySearchTextCriteria const&) const = 0;
+    virtual Vector<SimpleRange> findTextRanges(const AccessibilitySearchTextCriteria&) const = 0;
     virtual Vector<String> performTextOperation(AccessibilityTextOperation const&) = 0;
 
     virtual AXCoreObject* observableObject() const = 0;
@@ -841,7 +1149,7 @@ public:
     virtual void elementsFromAttribute(Vector<Element*>&, const QualifiedName&) const = 0;
 
     // Only if isColorWell()
-    virtual void colorValue(int& r, int& g, int& b) const = 0;
+    virtual SRGBA<uint8_t> colorValue() const = 0;
 
     virtual AccessibilityRole roleValue() const = 0;
     // Non-localized string associated with the object role.
@@ -866,6 +1174,7 @@ public:
 
     virtual TextIteratorBehavior textIteratorBehaviorForTextRange() const = 0;
     virtual PlainTextRange selectedTextRange() const = 0;
+    // FIXME: why do we need the following two methods if we already have selectedTextRange?
     virtual unsigned selectionStart() const = 0;
     virtual unsigned selectionEnd() const = 0;
 
@@ -878,6 +1187,7 @@ public:
     virtual PlatformWidget platformWidget() const = 0;
 #if PLATFORM(COCOA)
     virtual RemoteAXObjectRef remoteParentObject() const = 0;
+    virtual FloatRect convertRectToPlatformSpace(const FloatRect&, AccessibilityConversionSpace) const = 0;
 #endif
     virtual Widget* widgetForAttachmentView() const = 0;
     virtual Page* page() const = 0;
@@ -937,12 +1247,12 @@ public:
 
     virtual bool canHaveSelectedChildren() const = 0;
     virtual void selectedChildren(AccessibilityChildrenVector&) = 0;
+    virtual void setSelectedChildren(const AccessibilityChildrenVector&) = 0;
     virtual void visibleChildren(AccessibilityChildrenVector&) = 0;
     virtual void tabChildren(AccessibilityChildrenVector&) = 0;
     virtual bool shouldFocusActiveDescendant() const = 0;
     virtual AXCoreObject* activeDescendant() const = 0;
     virtual void handleActiveDescendantChanged() = 0;
-    virtual void handleAriaExpandedChanged() = 0;
     bool isDescendantOfObject(const AXCoreObject*) const;
     bool isAncestorOfObject(const AXCoreObject*) const;
     virtual AXCoreObject* firstAnonymousBlockChild() const = 0;
@@ -954,7 +1264,7 @@ public:
 
     virtual VisiblePositionRange visiblePositionRange() const = 0;
     virtual VisiblePositionRange visiblePositionRangeForLine(unsigned) const = 0;
-    virtual RefPtr<Range> elementRange() const = 0;
+    virtual Optional<SimpleRange> elementRange() const = 0;
     virtual VisiblePositionRange visiblePositionRangeForUnorderedPositions(const VisiblePosition&, const VisiblePosition&) const = 0;
     virtual VisiblePositionRange positionOfLeftWord(const VisiblePosition&) const = 0;
     virtual VisiblePositionRange positionOfRightWord(const VisiblePosition&) const = 0;
@@ -966,11 +1276,15 @@ public:
     virtual VisiblePositionRange visiblePositionRangeForRange(const PlainTextRange&) const = 0;
     virtual VisiblePositionRange lineRangeForPosition(const VisiblePosition&) const = 0;
 
-    virtual RefPtr<Range> rangeForPlainTextRange(const PlainTextRange&) const = 0;
+    virtual Optional<SimpleRange> rangeForPlainTextRange(const PlainTextRange&) const = 0;
+#if PLATFORM(MAC)
+    // FIXME: make this a COCOA method.
+    virtual AXTextMarkerRangeRef textMarkerRangeForNSRange(const NSRange&) const = 0;
+#endif
 
-    virtual String stringForRange(RefPtr<Range>) const = 0;
+    virtual String stringForRange(const SimpleRange&) const = 0;
     virtual IntRect boundsForVisiblePositionRange(const VisiblePositionRange&) const = 0;
-    virtual IntRect boundsForRange(const RefPtr<Range>) const = 0;
+    virtual IntRect boundsForRange(const SimpleRange&) const = 0;
     virtual int lengthForVisiblePositionRange(const VisiblePositionRange&) const = 0;
     virtual void setSelectedVisiblePositionRange(const VisiblePositionRange&) const = 0;
 
@@ -1161,11 +1475,14 @@ public:
     virtual String speechHintAttributeValue() const = 0;
     virtual String descriptionAttributeValue() const = 0;
     virtual String helpTextAttributeValue() const = 0;
+    // This should be the visible text that's actually on the screen if possible.
+    // If there's alternative text, that can override the title.
     virtual String titleAttributeValue() const = 0;
+
     virtual bool hasApplePDFAnnotationAttribute() const = 0;
 #endif
 
-#if PLATFORM(COCOA) && !PLATFORM(IOS_FAMILY)
+#if PLATFORM(MAC)
     virtual bool caretBrowsingEnabled() const = 0;
     virtual void setCaretBrowsingEnabled(bool) = 0;
 #endif
@@ -1186,6 +1503,9 @@ public:
     virtual String documentEncoding() const = 0;
     virtual AccessibilityChildrenVector documentLinks() = 0;
 
+    virtual String innerHTML() const = 0;
+    virtual String outerHTML() const = 0;
+
 private:
     // Detaches this object from the objects it references and it is referenced by.
     virtual void detachRemoteParts(AccessibilityDetachmentType) = 0;
@@ -1253,7 +1573,7 @@ template<typename U> inline void performFunctionOnMainThread(U&& lambda)
     if (isMainThread())
         return lambda();
 
-    callOnMainThread([&lambda] {
+    callOnMainThreadAndWait([&lambda] {
         lambda();
     });
 }
@@ -1301,6 +1621,9 @@ inline bool AXCoreObject::isAncestorOfObject(const AXCoreObject* axObject) const
 
 // Logging helpers.
 WTF::TextStream& operator<<(WTF::TextStream&, AccessibilityRole);
+WTF::TextStream& operator<<(WTF::TextStream&, AccessibilitySearchDirection);
+WTF::TextStream& operator<<(WTF::TextStream&, AccessibilitySearchKey);
+WTF::TextStream& operator<<(WTF::TextStream&, const AccessibilitySearchCriteria&);
 WTF::TextStream& operator<<(WTF::TextStream&, AccessibilityObjectInclusion);
 WTF::TextStream& operator<<(WTF::TextStream&, const AXCoreObject&);
 
