@@ -69,11 +69,11 @@ static bool isValidSimpleColor(StringView string)
 }
 
 // https://html.spec.whatwg.org/multipage/common-microsyntaxes.html#rules-for-parsing-simple-colour-values
-static Optional<SimpleColor> parseSimpleColorValue(StringView string)
+static Optional<SRGBA<uint8_t>> parseSimpleColorValue(StringView string)
 {
     if (!isValidSimpleColor(string))
         return WTF::nullopt;
-    return makeSimpleColor(toASCIIHexValue(string[1], string[2]), toASCIIHexValue(string[3], string[4]), toASCIIHexValue(string[5], string[6]));
+    return { { toASCIIHexValue(string[1], string[2]), toASCIIHexValue(string[3], string[4]), toASCIIHexValue(string[5], string[6]) } };
 }
 
 ColorInputType::~ColorInputType()
@@ -100,11 +100,6 @@ bool ColorInputType::isKeyboardFocusable(KeyboardEvent*) const
 #endif
 }
 
-bool ColorInputType::isColorControl() const
-{
-    return true;
-}
-
 bool ColorInputType::isPresentingAttachedView() const
 {
     return !!m_chooser;
@@ -139,7 +134,7 @@ Color ColorInputType::valueAsColor() const
     return parseSimpleColorValue(element()->value()).value();
 }
 
-void ColorInputType::createShadowSubtree()
+void ColorInputType::createShadowSubtreeAndUpdateInnerTextElementEditability(ContainerNode::ChildChange::Source source, bool)
 {
     ASSERT(element());
     ASSERT(element()->shadowRoot());
@@ -152,8 +147,8 @@ void ColorInputType::createShadowSubtree()
     wrapperElement->setPseudo(webkitColorSwatchWrapperName);
     auto colorSwatch = HTMLDivElement::create(document);
     colorSwatch->setPseudo(webkitColorSwatchName);
-    wrapperElement->appendChild(colorSwatch);
-    element()->userAgentShadowRoot()->appendChild(wrapperElement);
+    wrapperElement->appendChild(source, colorSwatch);
+    element()->userAgentShadowRoot()->appendChild(source, wrapperElement);
 
     updateColorSwatch();
 }
@@ -270,21 +265,6 @@ IntRect ColorInputType::elementRectRelativeToRootView() const
     return element()->document().view()->contentsToRootView(element()->renderer()->absoluteBoundingBoxRect());
 }
 
-Color ColorInputType::currentColor()
-{
-    return valueAsColor();
-}
-
-bool ColorInputType::shouldShowSuggestions() const
-{
-#if ENABLE(DATALIST_ELEMENT)
-    ASSERT(element());
-    return element()->hasAttributeWithoutSynchronization(listAttr);
-#else
-    return false;
-#endif
-}
-
 Vector<Color> ColorInputType::suggestedColors() const
 {
     Vector<Color> suggestions;
