@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,11 +30,12 @@
 #include "HTMLMediaElement.h"
 #include "Logging.h"
 #include "MediaPlayer.h"
+#include "NowPlayingInfo.h"
 #include "PlatformMediaSessionManager.h"
 
 namespace WebCore {
 
-static const Seconds clientDataBufferingTimerThrottleDelay { 100_ms };
+static constexpr Seconds clientDataBufferingTimerThrottleDelay { 100_ms };
 
 #if !RELEASE_LOG_DISABLED
 String convertEnumerationToString(PlatformMediaSession::State state)
@@ -65,6 +66,7 @@ String convertEnumerationToString(PlatformMediaSession::InterruptionType type)
         MAKE_STATIC_STRING_IMPL("SuspendedUnderLock"),
         MAKE_STATIC_STRING_IMPL("InvisibleAutoplay"),
         MAKE_STATIC_STRING_IMPL("ProcessInactive"),
+        MAKE_STATIC_STRING_IMPL("PlaybackSuspended"),
     };
     static_assert(!static_cast<size_t>(PlatformMediaSession::NoInterruption), "PlatformMediaSession::NoInterruption is not 0 as expected");
     static_assert(static_cast<size_t>(PlatformMediaSession::SystemSleep) == 1, "PlatformMediaSession::SystemSleep is not 1 as expected");
@@ -73,6 +75,7 @@ String convertEnumerationToString(PlatformMediaSession::InterruptionType type)
     static_assert(static_cast<size_t>(PlatformMediaSession::SuspendedUnderLock) == 4, "PlatformMediaSession::SuspendedUnderLock is not 4 as expected");
     static_assert(static_cast<size_t>(PlatformMediaSession::InvisibleAutoplay) == 5, "PlatformMediaSession::InvisibleAutoplay is not 5 as expected");
     static_assert(static_cast<size_t>(PlatformMediaSession::ProcessInactive) == 6, "PlatformMediaSession::ProcessInactive is not 6 as expected");
+    static_assert(static_cast<size_t>(PlatformMediaSession::PlaybackSuspended) == 7, "PlatformMediaSession::PlaybackSuspended is not 7 as expected");
     ASSERT(static_cast<size_t>(type) < WTF_ARRAY_LENGTH(values));
     return values[static_cast<size_t>(type)];
 }
@@ -90,6 +93,12 @@ String convertEnumerationToString(PlatformMediaSession::RemoteControlCommandType
         MAKE_STATIC_STRING_IMPL("BeginSeekingForwardCommand"),
         MAKE_STATIC_STRING_IMPL("EndSeekingForwardCommand"),
         MAKE_STATIC_STRING_IMPL("SeekToPlaybackPositionCommand"),
+        MAKE_STATIC_STRING_IMPL("SkipForwardCommand"),
+        MAKE_STATIC_STRING_IMPL("SkipBackwardCommand"),
+        MAKE_STATIC_STRING_IMPL("NextTrackCommand"),
+        MAKE_STATIC_STRING_IMPL("PreviousTrackCommand"),
+        MAKE_STATIC_STRING_IMPL("BeginScrubbing"),
+        MAKE_STATIC_STRING_IMPL("EndScrubbing"),
     };
     static_assert(!static_cast<size_t>(PlatformMediaSession::NoCommand), "PlatformMediaSession::NoCommand is not 0 as expected");
     static_assert(static_cast<size_t>(PlatformMediaSession::PlayCommand) == 1, "PlatformMediaSession::PlayCommand is not 1 as expected");
@@ -101,34 +110,40 @@ String convertEnumerationToString(PlatformMediaSession::RemoteControlCommandType
     static_assert(static_cast<size_t>(PlatformMediaSession::BeginSeekingForwardCommand) == 7, "PlatformMediaSession::BeginSeekingForwardCommand is not 7 as expected");
     static_assert(static_cast<size_t>(PlatformMediaSession::EndSeekingForwardCommand) == 8, "PlatformMediaSession::EndSeekingForwardCommand is not 8 as expected");
     static_assert(static_cast<size_t>(PlatformMediaSession::SeekToPlaybackPositionCommand) == 9, "PlatformMediaSession::SeekToPlaybackPositionCommand is not 9 as expected");
+    static_assert(static_cast<size_t>(PlatformMediaSession::SkipForwardCommand) == 10, "PlatformMediaSession::SkipForwardCommand is not 10 as expected");
+    static_assert(static_cast<size_t>(PlatformMediaSession::SkipBackwardCommand) == 11, "PlatformMediaSession::SkipBackwardCommand is not 11 as expected");
+    static_assert(static_cast<size_t>(PlatformMediaSession::NextTrackCommand) == 12, "PlatformMediaSession::NextTrackCommand is not 12 as expected");
+    static_assert(static_cast<size_t>(PlatformMediaSession::PreviousTrackCommand) == 13, "PlatformMediaSession::PreviousTrackCommand is not 13 as expected");
+    static_assert(static_cast<size_t>(PlatformMediaSession::BeginScrubbing) == 14, "PlatformMediaSession::BeginScrubbing is not 14 as expected");
+    static_assert(static_cast<size_t>(PlatformMediaSession::EndScrubbing) == 15, "PlatformMediaSession::EndScrubbing is not 15 as expected");
+
     ASSERT(static_cast<size_t>(command) < WTF_ARRAY_LENGTH(values));
     return values[static_cast<size_t>(command)];
 }
 
 #endif
 
-std::unique_ptr<PlatformMediaSession> PlatformMediaSession::create(PlatformMediaSessionClient& client)
+std::unique_ptr<PlatformMediaSession> PlatformMediaSession::create(PlatformMediaSessionManager& manager, PlatformMediaSessionClient& client)
 {
-    return makeUnique<PlatformMediaSession>(client);
+    return std::unique_ptr<PlatformMediaSession>(new PlatformMediaSession(manager, client));
 }
 
-PlatformMediaSession::PlatformMediaSession(PlatformMediaSessionClient& client)
-    : m_client(client)
-    , m_state(Idle)
-    , m_stateToRestore(Idle)
-    , m_notifyingClient(false)
+PlatformMediaSession::PlatformMediaSession(PlatformMediaSessionManager& manager, PlatformMediaSessionClient& client)
+    : m_manager(makeWeakPtr(manager))
+    , m_client(client)
+    , m_mediaSessionIdentifier(MediaSessionIdentifier::generate())
 #if !RELEASE_LOG_DISABLED
-    , m_logger(client.hostingDocument()->logger())
+    , m_logger(client.logger())
     , m_logIdentifier(uniqueLogIdentifier())
 #endif
 {
-    ASSERT(m_client.mediaType() >= None && m_client.mediaType() <= MediaStreamCapturingAudio);
-    PlatformMediaSessionManager::sharedManager().addSession(*this);
+    manager.addSession(*this);
 }
 
 PlatformMediaSession::~PlatformMediaSession()
 {
-    PlatformMediaSessionManager::sharedManager().removeSession(*this);
+    if (m_manager)
+        m_manager->removeSession(*this);
 }
 
 void PlatformMediaSession::setState(State state)
@@ -136,16 +151,16 @@ void PlatformMediaSession::setState(State state)
     if (state == m_state)
         return;
 
-    INFO_LOG(LOGIDENTIFIER, state);
+    ALWAYS_LOG(LOGIDENTIFIER, state);
     m_state = state;
     if (m_state == State::Playing)
         m_hasPlayedSinceLastInterruption = true;
-    PlatformMediaSessionManager::sharedManager().sessionStateChanged(*this);
+    m_manager->sessionStateChanged(*this);
 }
 
 void PlatformMediaSession::beginInterruption(InterruptionType type)
 {
-    INFO_LOG(LOGIDENTIFIER, "state = ", m_state, ", interruption type = ", type, ", interruption count = ", m_interruptionCount);
+    ALWAYS_LOG(LOGIDENTIFIER, "state = ", m_state, ", interruption type = ", type, ", interruption count = ", m_interruptionCount);
 
     // When interruptions are overridden, m_interruptionType doesn't get set.
     // Give nested interruptions a chance when the previous interruptions were overridden.
@@ -153,7 +168,7 @@ void PlatformMediaSession::beginInterruption(InterruptionType type)
         return;
 
     if (client().shouldOverrideBackgroundPlaybackRestriction(type)) {
-        INFO_LOG(LOGIDENTIFIER, "returning early because client says to override interruption");
+        ALWAYS_LOG(LOGIDENTIFIER, "returning early because client says to override interruption");
         return;
     }
 
@@ -167,10 +182,10 @@ void PlatformMediaSession::beginInterruption(InterruptionType type)
 
 void PlatformMediaSession::endInterruption(EndInterruptionFlags flags)
 {
-    INFO_LOG(LOGIDENTIFIER, "flags = ", (int)flags, ", stateToRestore = ", m_stateToRestore, ", interruption count = ", m_interruptionCount);
+    ALWAYS_LOG(LOGIDENTIFIER, "flags = ", (int)flags, ", stateToRestore = ", m_stateToRestore, ", interruption count = ", m_interruptionCount);
 
     if (!m_interruptionCount) {
-        INFO_LOG(LOGIDENTIFIER, "!! ignoring spurious interruption end !!");
+        ALWAYS_LOG(LOGIDENTIFIER, "!! ignoring spurious interruption end !!");
         return;
     }
 
@@ -197,10 +212,10 @@ void PlatformMediaSession::clientWillBeginAutoplaying()
     if (m_notifyingClient)
         return;
 
-    INFO_LOG(LOGIDENTIFIER, "state = ", m_state);
+    ALWAYS_LOG(LOGIDENTIFIER, "state = ", m_state);
     if (state() == Interrupted) {
         m_stateToRestore = Autoplaying;
-        INFO_LOG(LOGIDENTIFIER, "      setting stateToRestore to \"Autoplaying\"");
+        ALWAYS_LOG(LOGIDENTIFIER, "      setting stateToRestore to \"Autoplaying\"");
         return;
     }
 
@@ -212,9 +227,9 @@ bool PlatformMediaSession::clientWillBeginPlayback()
     if (m_notifyingClient)
         return true;
 
-    INFO_LOG(LOGIDENTIFIER, "state = ", m_state);
+    ALWAYS_LOG(LOGIDENTIFIER, "state = ", m_state);
 
-    if (!PlatformMediaSessionManager::sharedManager().sessionWillBeginPlayback(*this)) {
+    if (!m_manager->sessionWillBeginPlayback(*this)) {
         if (state() == Interrupted)
             m_stateToRestore = Playing;
         return false;
@@ -224,34 +239,46 @@ bool PlatformMediaSession::clientWillBeginPlayback()
     return true;
 }
 
-bool PlatformMediaSession::clientWillPausePlayback()
+bool PlatformMediaSession::processClientWillPausePlayback(DelayCallingUpdateNowPlaying shouldDelayCallingUpdateNowPlaying)
 {
     if (m_notifyingClient)
         return true;
 
-    INFO_LOG(LOGIDENTIFIER, "state = ", m_state);
+    ALWAYS_LOG(LOGIDENTIFIER, "state = ", m_state);
     if (state() == Interrupted) {
         m_stateToRestore = Paused;
-        INFO_LOG(LOGIDENTIFIER, "      setting stateToRestore to \"Paused\"");
+        ALWAYS_LOG(LOGIDENTIFIER, "      setting stateToRestore to \"Paused\"");
         return false;
     }
     
     setState(Paused);
-    PlatformMediaSessionManager::sharedManager().sessionWillEndPlayback(*this);
+    m_manager->sessionWillEndPlayback(*this, shouldDelayCallingUpdateNowPlaying);
     return true;
 }
 
+bool PlatformMediaSession::clientWillPausePlayback()
+{
+    ALWAYS_LOG(LOGIDENTIFIER);
+    return processClientWillPausePlayback(DelayCallingUpdateNowPlaying::No);
+}
+
+void PlatformMediaSession::clientWillBeDOMSuspended()
+{
+    ALWAYS_LOG(LOGIDENTIFIER);
+    processClientWillPausePlayback(DelayCallingUpdateNowPlaying::Yes);
+}
+
 void PlatformMediaSession::pauseSession()
 {
-    INFO_LOG(LOGIDENTIFIER);
+    ALWAYS_LOG(LOGIDENTIFIER);
     m_client.suspendPlayback();
 }
 
 void PlatformMediaSession::stopSession()
 {
-    INFO_LOG(LOGIDENTIFIER);
+    ALWAYS_LOG(LOGIDENTIFIER);
     m_client.suspendPlayback();
-    PlatformMediaSessionManager::sharedManager().removeSession(*this);
+    m_manager->removeSession(*this);
 }
 
 PlatformMediaSession::MediaType PlatformMediaSession::mediaType() const
@@ -264,41 +291,14 @@ PlatformMediaSession::MediaType PlatformMediaSession::presentationType() const
     return m_client.presentationType();
 }
 
-PlatformMediaSession::CharacteristicsFlags PlatformMediaSession::characteristics() const
-{
-    return m_client.characteristics();
-}
-
-#if ENABLE(VIDEO)
-uint64_t PlatformMediaSession::uniqueIdentifier() const
-{
-    return m_client.mediaSessionUniqueIdentifier();
-}
-
-String PlatformMediaSession::title() const
-{
-    return m_client.mediaSessionTitle();
-}
-
-double PlatformMediaSession::duration() const
-{
-    return m_client.mediaSessionDuration();
-}
-
-double PlatformMediaSession::currentTime() const
-{
-    return m_client.mediaSessionCurrentTime();
-}
-#endif
-    
 bool PlatformMediaSession::canReceiveRemoteControlCommands() const
 {
     return m_client.canReceiveRemoteControlCommands();
 }
 
-void PlatformMediaSession::didReceiveRemoteControlCommand(RemoteControlCommandType command, const PlatformMediaSession::RemoteCommandArgument* argument)
+void PlatformMediaSession::didReceiveRemoteControlCommand(RemoteControlCommandType command, const PlatformMediaSession::RemoteCommandArgument& argument)
 {
-    INFO_LOG(LOGIDENTIFIER, command);
+    ALWAYS_LOG(LOGIDENTIFIER, command);
 
     m_client.didReceiveRemoteControlCommand(command, argument);
 }
@@ -308,11 +308,6 @@ bool PlatformMediaSession::supportsSeeking() const
     return m_client.supportsSeeking();
 }
 
-String PlatformMediaSession::sourceApplicationIdentifier() const
-{
-    return m_client.sourceApplicationIdentifier();
-}
-
 bool PlatformMediaSession::isSuspended() const
 {
     return m_client.isSuspended();
@@ -333,7 +328,7 @@ void PlatformMediaSession::isPlayingToWirelessPlaybackTargetChanged(bool isWirel
     // Save and restore the interruption count so it doesn't get out of sync if beginInterruption is called because
     // if we in the background.
     int interruptionCount = m_interruptionCount;
-    PlatformMediaSessionManager::sharedManager().sessionIsPlayingToWirelessPlaybackTargetChanged(*this);
+    m_manager->sessionIsPlayingToWirelessPlaybackTargetChanged(*this);
     m_interruptionCount = interruptionCount;
 }
 
@@ -344,7 +339,7 @@ PlatformMediaSession::DisplayType PlatformMediaSession::displayType() const
 
 bool PlatformMediaSession::activeAudioSessionRequired() const
 {
-    if (mediaType() == PlatformMediaSession::None)
+    if (mediaType() == PlatformMediaSession::MediaType::None)
         return false;
     if (state() != PlatformMediaSession::State::Playing)
         return false;
@@ -358,39 +353,32 @@ bool PlatformMediaSession::canProduceAudio() const
 
 void PlatformMediaSession::canProduceAudioChanged()
 {
-    PlatformMediaSessionManager::sharedManager().sessionCanProduceAudioChanged();
+    m_manager->sessionCanProduceAudioChanged();
 }
 
-#if ENABLE(VIDEO)
-uint64_t PlatformMediaSessionClient::mediaSessionUniqueIdentifier() const
-{
-    return 0;
-}
-
-String PlatformMediaSessionClient::mediaSessionTitle() const
-{
-    return String();
-}
-
-double PlatformMediaSessionClient::mediaSessionDuration() const
-{
-    return MediaPlayer::invalidTime();
-}
-
-double PlatformMediaSessionClient::mediaSessionCurrentTime() const
-{
-    return MediaPlayer::invalidTime();
-}
-#endif
-
 void PlatformMediaSession::clientCharacteristicsChanged()
 {
-    PlatformMediaSessionManager::sharedManager().clientCharacteristicsChanged(*this);
+    m_manager->clientCharacteristicsChanged(*this);
+}
+
+static inline bool isPlayingAudio(PlatformMediaSession::MediaType mediaType)
+{
+    return mediaType == MediaElementSession::MediaType::VideoAudio || mediaType == MediaElementSession::MediaType::Audio;
 }
 
 bool PlatformMediaSession::canPlayConcurrently(const PlatformMediaSession& otherSession) const
 {
-    return m_client.hasMediaStreamSource() && otherSession.m_client.hasMediaStreamSource();
+    auto mediaType = this->mediaType();
+    auto otherMediaType = otherSession.mediaType();
+    if (otherMediaType != mediaType && (!isPlayingAudio(mediaType) || !isPlayingAudio(otherMediaType)))
+        return true;
+
+    auto groupID = client().mediaSessionGroupIdentifier();
+    auto otherGroupID = otherSession.client().mediaSessionGroupIdentifier();
+    if (!groupID || !otherGroupID || groupID != otherGroupID)
+        return false;
+
+    return m_client.hasMediaStreamSource() || otherSession.m_client.hasMediaStreamSource();
 }
 
 bool PlatformMediaSession::shouldOverridePauseDuringRouteChange() const
@@ -398,6 +386,16 @@ bool PlatformMediaSession::shouldOverridePauseDuringRouteChange() const
     return m_client.shouldOverridePauseDuringRouteChange();
 }
 
+PlatformMediaSessionManager& PlatformMediaSession::manager()
+{
+    return *m_manager;
+}
+
+Optional<NowPlayingInfo> PlatformMediaSession::nowPlayingInfo() const
+{
+    return { };
+}
+
 #if !RELEASE_LOG_DISABLED
 WTFLogChannel& PlatformMediaSession::logChannel() const
 {
