@@ -23,27 +23,12 @@
 #if USE(GSTREAMER)
 #include <gst/gst.h>
 
+#if USE(GSTREAMER_GL)
+#include <gst/gl/egl/gsteglimage.h>
+#endif
+
 namespace WTF {
 
-template<> GRefPtr<GstMiniObject> adoptGRef(GstMiniObject* ptr)
-{
-    return GRefPtr<GstMiniObject>(ptr, GRefPtrAdopt);
-}
-
-template<> GstMiniObject* refGPtr<GstMiniObject>(GstMiniObject* ptr)
-{
-    if (ptr)
-        gst_mini_object_ref(ptr);
-
-    return ptr;
-}
-
-template<> void derefGPtr<GstMiniObject>(GstMiniObject* ptr)
-{
-    if (ptr)
-        gst_mini_object_unref(ptr);
-}
-
 template <> GRefPtr<GstElement> adoptGRef(GstElement* ptr)
 {
     ASSERT(!ptr || !g_object_is_floating(ptr));
@@ -64,6 +49,27 @@ template <> void derefGPtr<GstElement>(GstElement* ptr)
         gst_object_unref(ptr);
 }
 
+
+template <> GRefPtr<GstPlugin> adoptGRef(GstPlugin* ptr)
+{
+    ASSERT(!ptr || !g_object_is_floating(ptr));
+    return GRefPtr<GstPlugin>(ptr, GRefPtrAdopt);
+}
+
+template <> GstPlugin* refGPtr<GstPlugin>(GstPlugin* ptr)
+{
+    if (ptr)
+        gst_object_ref_sink(GST_OBJECT(ptr));
+
+    return ptr;
+}
+
+template <> void derefGPtr<GstPlugin>(GstPlugin* ptr)
+{
+    if (ptr)
+        gst_object_unref(ptr);
+}
+
 template <> GRefPtr<GstPad> adoptGRef(GstPad* ptr)
 {
     ASSERT(!ptr || !g_object_is_floating(ptr));
@@ -410,6 +416,28 @@ template <> void derefGPtr<GstStreamCollection>(GstStreamCollection* ptr)
         gst_object_unref(ptr);
 }
 
+template <>
+GRefPtr<GstClock> adoptGRef(GstClock* ptr)
+{
+    return GRefPtr<GstClock>(ptr, GRefPtrAdopt);
+}
+
+template <>
+GstClock* refGPtr<GstClock>(GstClock* ptr)
+{
+    if (ptr)
+        gst_object_ref(GST_OBJECT_CAST(ptr));
+
+    return ptr;
+}
+
+template <>
+void derefGPtr<GstClock>(GstClock* ptr)
+{
+    if (ptr)
+        gst_object_unref(ptr);
+}
+
 template <> GRefPtr<WebKitVideoSink> adoptGRef(WebKitVideoSink* ptr)
 {
     ASSERT(!ptr || !g_object_is_floating(ptr));
@@ -502,6 +530,24 @@ template<> void derefGPtr<GstGLContext>(GstGLContext* ptr)
         gst_object_unref(GST_OBJECT(ptr));
 }
 
+template <> GRefPtr<GstEGLImage> adoptGRef(GstEGLImage* ptr)
+{
+    return GRefPtr<GstEGLImage>(ptr, GRefPtrAdopt);
+}
+
+template <> GstEGLImage* refGPtr<GstEGLImage>(GstEGLImage* ptr)
+{
+    if (ptr)
+        gst_egl_image_ref(ptr);
+    return ptr;
+}
+
+template <> void derefGPtr<GstEGLImage>(GstEGLImage* ptr)
+{
+    if (ptr)
+        gst_egl_image_unref(ptr);
+}
+
 #endif // USE(GSTREAMER_GL)
 
 } // namespace WTF
