@@ -28,6 +28,7 @@
 #include "ImageGStreamer.h"
 #include "MediaSampleGStreamer.h"
 #include "NotImplemented.h"
+#include "RuntimeApplicationChecks.h"
 #include <gst/app/gstappsink.h>
 #include <wtf/Lock.h>
 #include <wtf/MainThread.h>
@@ -46,11 +47,11 @@ public:
         return adoptRef(*new ImageDecoderGStreamerSample(WTFMove(sample), presentationSize));
     }
 
-    NativeImagePtr image() const
+    PlatformImagePtr image() const
     {
         if (!m_image)
             return nullptr;
-        return m_image->image().nativeImage();
+        return m_image->image().nativeImage()->platformImage();
     }
     void dropImage() { m_image = nullptr; }
 
@@ -98,7 +99,15 @@ ImageDecoderGStreamer::ImageDecoderGStreamer(SharedBuffer& data, const String& m
 
 bool ImageDecoderGStreamer::supportsContainerType(const String& type)
 {
-    return GStreamerRegistryScanner::singleton().isContainerTypeSupported(type);
+    // Ideally this decoder should operate only from the WebProcess (or from the GPUProcess) which
+    // should be the only process where GStreamer has been runtime initialized.
+    if (!isInWebProcess())
+        return false;
+
+    if (!type.startsWith("video/"_s))
+        return false;
+
+    return GStreamerRegistryScanner::singleton().isContainerTypeSupported(GStreamerRegistryScanner::Configuration::Decoding, type);
 }
 
 bool ImageDecoderGStreamer::canDecodeType(const String& mimeType)
@@ -106,7 +115,15 @@ bool ImageDecoderGStreamer::canDecodeType(const String& mimeType)
     if (mimeType.isEmpty())
         return false;
 
-    return GStreamerRegistryScanner::singleton().isContainerTypeSupported(mimeType);
+    if (!mimeType.startsWith("video/"_s))
+        return false;
+
+    // Ideally this decoder should operate only from the WebProcess (or from the GPUProcess) which
+    // should be the only process where GStreamer has been runtime initialized.
+    if (!isInWebProcess())
+        return false;
+
+    return GStreamerRegistryScanner::singleton().isContainerTypeSupported(GStreamerRegistryScanner::Configuration::Decoding, mimeType);
 }
 
 EncodedDataStatus ImageDecoderGStreamer::encodedDataStatus() const
@@ -139,10 +156,10 @@ String ImageDecoderGStreamer::uti() const
     return { };
 }
 
-ImageOrientation ImageDecoderGStreamer::frameOrientationAtIndex(size_t) const
+ImageDecoder::FrameMetadata ImageDecoderGStreamer::frameMetadataAtIndex(size_t) const
 {
     notImplemented();
-    return ImageOrientation::None;
+    return { };
 }
 
 Seconds ImageDecoderGStreamer::frameDurationAtIndex(size_t index) const
@@ -169,7 +186,7 @@ unsigned ImageDecoderGStreamer::frameBytesAtIndex(size_t index, SubsamplingLevel
     return (frameSize.area() * 4).unsafeGet();
 }
 
-NativeImagePtr ImageDecoderGStreamer::createFrameImageAtIndex(size_t index, SubsamplingLevel, const DecodingOptions&)
+PlatformImagePtr ImageDecoderGStreamer::createFrameImageAtIndex(size_t index, SubsamplingLevel, const DecodingOptions&)
 {
     LockHolder holder { m_sampleGeneratorLock };
 
@@ -220,7 +237,7 @@ void ImageDecoderGStreamer::InnerDecoder::decodebinPadAddedCallback(ImageDecoder
 
 void ImageDecoderGStreamer::InnerDecoder::connectDecoderPad(GstPad* pad)
 {
-    auto padCaps = adoptGRef(gst_pad_get_current_caps(pad));
+    auto padCaps = adoptGRef(gst_pad_query_caps(pad, nullptr));
     GST_DEBUG_OBJECT(m_pipeline.get(), "New decodebin pad %" GST_PTR_FORMAT " caps: %" GST_PTR_FORMAT, pad, padCaps.get());
     RELEASE_ASSERT(doCapsHaveType(padCaps.get(), "video"));
 
@@ -301,6 +318,27 @@ void ImageDecoderGStreamer::InnerDecoder::handleMessage(GstMessage* message)
         g_warning("Error: %d, %s. Debug output: %s", error->code, error->message, debug.get());
         m_decoder.setHasEOS();
         break;
+    case GST_MESSAGE_STREAM_COLLECTION: {
+        GRefPtr<GstStreamCollection> collection;
+        gst_message_parse_stream_collection(message, &collection.outPtr());
+        if (collection) {
+            unsigned size = gst_stream_collection_get_size(collection.get());
+            GList* streams = nullptr;
+            for (unsigned i = 0 ; i < size; i++) {
+                auto* stream = gst_stream_collection_get_stream(collection.get(), i);
+                auto streamType = gst_stream_get_stream_type(stream);
+                if (streamType == GST_STREAM_TYPE_VIDEO) {
+                    streams = g_list_append(streams, const_cast<char*>(gst_stream_get_stream_id(stream)));
+                    break;
+                }
+            }
+            if (streams) {
+                gst_element_send_event(m_decodebin.get(), gst_event_new_select_streams(streams));
+                g_list_free(streams);
+            }
+        }
+        break;
+    }
     default:
         break;
     }
@@ -333,13 +371,11 @@ void ImageDecoderGStreamer::InnerDecoder::preparePipeline()
     GstElement* source = gst_element_factory_make("giostreamsrc", nullptr);
     g_object_set(source, "stream", m_memoryStream.get(), nullptr);
 
-    GstElement* decoder = gst_element_factory_make("decodebin", nullptr);
-    auto allowedCaps = adoptGRef(gst_caps_new_empty_simple("video/x-raw"));
-    g_object_set(decoder, "caps", allowedCaps.get(), "expose-all-streams", false, nullptr);
-    g_signal_connect_swapped(decoder, "pad-added", G_CALLBACK(decodebinPadAddedCallback), this);
+    m_decodebin = gst_element_factory_make("decodebin3", nullptr);
+    g_signal_connect_swapped(m_decodebin.get(), "pad-added", G_CALLBACK(decodebinPadAddedCallback), this);
 
-    gst_bin_add_many(GST_BIN_CAST(m_pipeline.get()), source, decoder, nullptr);
-    gst_element_link(source, decoder);
+    gst_bin_add_many(GST_BIN_CAST(m_pipeline.get()), source, m_decodebin.get(), nullptr);
+    gst_element_link(source, m_decodebin.get());
     gst_element_set_state(m_pipeline.get(), GST_STATE_PLAYING);
 }
 
