@@ -252,7 +252,6 @@ private:
         }
 
         webkitURIRequestGetResourceRequest(request.get(), resourceRequest);
-        resourceRequest.setInitiatingPageID(page.webPageProxyIdentifier().toUInt64());
 
         API::Dictionary::MapType message;
         message.set(String::fromUTF8("Page"), &page);
@@ -746,7 +745,7 @@ void webkitWebPageDidReceiveMessage(WebKitWebPage* page, const String& messageNa
                 Color savedBackgroundColor;
                 if (transparentBackground) {
                     savedBackgroundColor = frameView->baseBackgroundColor();
-                    frameView->setBaseBackgroundColor(Color::transparent);
+                    frameView->setBaseBackgroundColor(Color::transparentBlack);
                 }
                 snapshotImage = webPage->scaledSnapshotWithOptions(snapshotRect, 1, snapshotOptions | SnapshotOptionsShareable);
                 if (transparentBackground)
